//
//  ThumbnailViewAppDelegate.m
//  ThumbnailView
//
//  Created by 國居 貴浩 on 11/11/19.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "ThumbnailViewAppDelegate.h"

#import "ThumbnailViewController.h"

@implementation ThumbnailViewAppDelegate

@synthesize window = _window;
@synthesize viewController = _viewController;

- (void)dealloc
{
    [_window release];
    [_viewController release];
    [super dealloc];
}

/*
 グラデーションでの塗りつぶし。色相をhueで受け取ってグラデーションに使う。
 */
- (void)fillGradation:(CGContextRef)context startPoint:(CGPoint)gradientStartPoint endPoint:(CGPoint)gradientEndPoint hue:(float)hue
{
    CGColorRef c1 = [UIColor colorWithHue:hue saturation:1 brightness:1 alpha:1].CGColor;
    CGColorRef c2 = [UIColor colorWithHue:hue saturation:1 brightness:0.2 alpha:1].CGColor;
    NSArray* colors = [NSArray arrayWithObjects:(id)c1, (id)c2, nil];
    CGFloat locations[] = {0.0, 1.0};
    CGGradientRef gradient = CGGradientCreateWithColors(CGColorGetColorSpace(c1), (CFArrayRef)colors, locations);
    CGContextDrawLinearGradient(context, gradient, 
                                gradientStartPoint, gradientEndPoint, kCGGradientDrawsAfterEndLocation);
    CGGradientRelease(gradient);
}

/*
 1 x 1の座標系でハートパス定義
 */
- (void)buildNormalHeart:(CGContextRef)context
{
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, 0, 0.25);
    CGContextAddCurveToPoint(context, 0.1, 0, 0.5, 0, 0.5, 0.3);
    CGContextAddCurveToPoint(context, 0.5, 0.75, 0, 0.9, 0, 0.9);
    CGContextAddCurveToPoint(context, 0, 0.9, -0.5, 0.75, -0.5, 0.3);
    CGContextAddCurveToPoint(context, -0.5, 0, -0.1, 0, 0, 0.25);
    CGContextClosePath(context);    
}

/*
 frameで指定される位置にハート描画。
 */
- (void)drawHeart:(CGContextRef)context frame:(CGRect)frame
{
    //  座標系を1 x 1に変更。
    CGContextTranslateCTM(context, frame.origin.x + frame.size.width / 2.0, frame.origin.y);
    CGContextScaleCTM(context, frame.size.width, frame.size.height);
    [self buildNormalHeart:context];
    CGContextClip(context);
}

/*
 ダミー画像作成。
 */
- (UIImage*)buildDummyImage:(CGSize)size hue:(float)hue
{
    UIGraphicsBeginImageContext(size);    
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    //  ハートを描く。
    [self drawHeart:context frame:CGRectMake(0, 0, size.width, size.height)];
    [self fillGradation:context startPoint:CGPointMake(0,1) endPoint:CGPointMake(0,0) hue:hue];
    
    UIImage* image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return image;
}

- (NSString*)imageDirectory
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString* imageDirectory = [documentsDirectory stringByAppendingPathComponent:@"images"];
    return imageDirectory;
}

- (void)buildDummys
{    
    //  Documents/imagesディレクトリ作成。
    NSString* imageDirectory = [self imageDirectory];
    [[NSFileManager defaultManager] createDirectoryAtPath:imageDirectory withIntermediateDirectories:YES attributes:0 error:nil];
    for (int i = 0; i < 200; i++) { 
        //  Documents/imagesディレクトリ内をtest0.png、test1.png…と順に検索。
        NSString* path = [imageDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"test%d.png", i]];
        if ([[NSFileManager defaultManager] fileExistsAtPath:path] == NO) {
            UIImage* image = [self buildDummyImage:CGSizeMake(1000, 1000) hue:(float)(i % 30) / 30.0];
            NSData* data = UIImagePNGRepresentation(image);
            [data writeToFile:path atomically:YES];
        }
    }
    printf("path = [%s]\n", [imageDirectory UTF8String]);
}

- (NSArray*)images
{
    NSString* imageDirectory = [self imageDirectory];
    NSError* error = nil;
    NSArray* names = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:imageDirectory error:&error];
    if (error) {
        NSLog(@"%@", error);
        return nil;
    }
    NSMutableArray* images = [NSMutableArray arrayWithCapacity:[names count]];
    for (NSString* name in names) {
        [images addObject:[imageDirectory stringByAppendingPathComponent:name]];
    }
    return images;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    [self buildDummys];
    self.window = [[[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]] autorelease];
    
    ThumbnailViewController* thumbnailViewController = [[[ThumbnailViewController alloc] initWithNibName:@"ThumbnailViewController" bundle:nil] autorelease];
    thumbnailViewController.images = [self images];
    // Override point for customization after application launch.
    self.viewController = thumbnailViewController;
    self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    /*
     Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
     If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
     */
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    /*
     Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
     */
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    /*
     Called when the application is about to terminate.
     Save data if appropriate.
     See also applicationDidEnterBackground:.
     */
}

@end
