//
//  ThumbnailViewViewController.m
//  ThumbnailView
//
//  Created by 國居 貴浩 on 11/11/19.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "ThumbnailViewController.h"

@interface ThumbTileView : UIScrollView {
    NSArray* _images;   //  サムネイル元画像ファイルパスの配列
}
@end

@implementation ThumbTileView

//  渡されたimagesとthumbSizeでUIImageViewをタイル状に敷き詰める。
- (void)setImages:(NSArray*)images thumbSize:(CGSize)thumbSize
{
    //  以前貼られていたUIImageViewをすべて取りはぶく
    NSArray* views = [self.subviews copy];
    for (UIView* view in views) {
        [view removeFromSuperview];
    }
    [views release];
    
    //  新しいUIImageViewの敷き詰め準備
    _images = images;   //  ThumbnailViewController側でretainしてるので、こちらはretainしなくても特に問題はない。
    CGRect r = CGRectMake(0, 0, thumbSize.width, thumbSize.height);
    int max = [_images count];
    
    //  必要なcontentSizeの計算
    int rowCount = (int)self.bounds.size.width / (int)thumbSize.width;
    int vCount = (max + rowCount - 1) / rowCount;
    self.contentSize = CGSizeMake(self.bounds.size.width, vCount * thumbSize.height);
    
    //  必要なUIImageViewを敷き詰める。
    for (int tag = 1; tag <= max; tag++) {
        UIImageView* imageview = [[[UIImageView alloc]initWithFrame:CGRectInset(r, 4, 4)]autorelease];
        imageview.tag = tag;
        [self addSubview:imageview];
        r = CGRectOffset(r, r.size.width, 0);
        if ((r.origin.x + r.size.width) > self.bounds.size.width) {
            r.origin.x = 0;
            r = CGRectOffset(r, 0, r.size.height);
        }
    }    
}

-(UIImage*)resizeImage:(UIImage*)image size:(CGSize)size
{
    //  指定されたサイズのオフスクリーン作成。
    UIGraphicsBeginImageContext(size);
    //  オフスクリーンいっぱいにimageを描画。
    [image drawInRect:CGRectMake(0, 0, size.width, size.height)];
    //  オフスクリーンからUIImage作成。
    UIImage* resizedImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return resizedImage;    
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    if (_images == nil) {
        return;
    }
    CGRect visibleFrame = CGRectMake(self.contentOffset.x, self.contentOffset.y, self.bounds.size.width, self.bounds.size.height);
    int tag = 1;
    for (NSString* filepath in _images) {
        UIImageView* imageview = (UIImageView*)[self viewWithTag:tag++];
        if (CGRectIntersectsRect(visibleFrame, imageview.frame) == NO) {
            continue;
        }
        if (imageview.image) {
            continue;
        }
        UIImage* image = [UIImage imageWithContentsOfFile:filepath];
        imageview.image = [self resizeImage:image size:imageview.bounds.size];
    }
}
@end

@implementation ThumbnailViewController {
    UIActivityIndicatorView* _activityIndicatorView;
    ThumbTileView* _baseview;
}
@synthesize images;

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

-(UIImage*)resizeImage:(UIImage*)image size:(CGSize)size
{
    //  指定されたサイズのオフスクリーン作成。
    UIGraphicsBeginImageContext(size);
    //  オフスクリーンいっぱいにimageを描画。
    [image drawInRect:CGRectMake(0, 0, size.width, size.height)];
    //  オフスクリーンからUIImage作成。
    UIImage* resizedImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return resizedImage;    
}

//  画像読み込み、貼付け処理
- (void)loadingimages
{
    [[UIApplication sharedApplication] endIgnoringInteractionEvents];
    CGRect r = CGRectMake(0, 0, 44, 44);
    for (NSString* filepath in images) {
        UIImage* image = [UIImage imageWithContentsOfFile:filepath];
        UIImageView* view = [[[UIImageView alloc]initWithFrame:CGRectInset(r, 4, 4)]autorelease];
        view.image = [self resizeImage:image size:view.bounds.size];
        [_baseview addSubview:view];
        r = CGRectOffset(r, r.size.width, 0);
        if (r.origin.x > _baseview.bounds.size.width) {
            r.origin.x = 0;
            r = CGRectOffset(r, 0, r.size.height);
        }
    }    
    [_activityIndicatorView stopAnimating];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //  先に画像用のベース作成、貼付け
    _baseview = [[[ThumbTileView alloc] initWithFrame:self.view.bounds]autorelease];
    [_baseview setImages:images thumbSize:CGSizeMake(100, 100)];
    [self.view addSubview:_baseview];
    
    //  インジケータ貼付け
    _activityIndicatorView = [[[UIActivityIndicatorView alloc] initWithFrame:CGRectMake((self.view.bounds.size.width - 44) / 2, 100, 44, 44)]autorelease];
    [self.view addSubview:_activityIndicatorView];

    return;
    
    //  グルグル開始
    [_activityIndicatorView startAnimating];

    //  画像読み込み処理時間差実行
    [NSObject cancelPreviousPerformRequestsWithTarget:self];
    [[UIApplication sharedApplication] beginIgnoringInteractionEvents];
    [self performSelector:@selector(loadingimages) withObject:nil afterDelay:0];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

@end
